@@ -56,9 +56,12 @@ class HC(object) :
         self._imin_jin = np.argmax(self._pvals > np.log(self._N) / self._N)
 
     def _calculateHC(self, imin, imax) :
-        if imin > imax :
+        if imin > imax:
             return np.nan
-        self._istar = np.argmax(self._zz[imin:imax]) + imin
+        if imin==imax:
+            self._istar = imin
+        else: 
+            self._istar = np.argmax(self._zz[imin:imax]) + imin
         zMaxStar = self._zz[self._istar]
         return zMaxStar, self._pvals[self._istar]
 
@@ -70,14 +73,13 @@ class HC(object) :
         -----
         'gamma' : lower fraction of P-values to consider
 
-        Returns:
+        Return:
         -------
         HC test score, P-value attaining it
 
         """
         imin = 0
-        imax = np.maximum(imin + 1,
-                        int(np.floor(gamma * self._N + 0.5)))        
+        imax = np.maximum(imin, int(gamma * self._N + 0.5))        
         return self._calculateHC(imin, imax)
 
     def HCjin(self, gamma=0.2) :
@@ -87,7 +89,7 @@ class HC(object) :
         -----
         'gamma' : lower fraction of P-values to consider
 
-        Returns:
+        Return:
         -------
         HC score, P-value attaining it
 
@@ -109,7 +111,7 @@ class HC(object) :
         -----
         'gamma' : lower fraction of P-values to consider
 
-        Returns:
+        Return:
         -------
         -log(BJ) score (large values are significant) 
         (has a scaled chisquared distribution under the null)
@@ -123,7 +125,7 @@ class HC(object) :
     
         max_i = max(1, int(gamma * N))
 
-        spv = spv = self._pvals[:max_i]
+        spv = self._pvals[:max_i]
         ii = np.arange(1, max_i+1)
 
         bj = spv[0]
@@ -135,7 +137,8 @@ class HC(object) :
             Mminus = np.min(1 - BJpv)
             bj = np.minimum(Mplus, Mminus)
             
-        return -np.log(bj)
+        return -np.log(bj + self._EPS / 1000) # we add a tiny amount to avoide
+                                              # divide by zero warning when bj ~ 0
 
 
     def HCstar(self, gamma=0.2) :
@@ -156,7 +159,7 @@ class HC(object) :
                         int(np.floor(gamma * self._N + 0.5)))        
         return self._calculateHC(imin, imax)
 
-    def get_state(self) :
+    def get_state(self):
         return {'pvals' : self._pvals, 
                 'u' : self._uu,
                 'z' : self._zz,
@@ -164,6 +167,21 @@ class HC(object) :
                 'imin_jin' : self._imin_jin,
                 }
 
+    def Bonf(self):
+        """
+        Bonferroni type inference
+        """
+        return -np.log(self._pvals[0])
+
+    def FDR(self):
+        """
+        False-discovery rate thresholding
+        """
+        vals = self._pvals / self._uu
+        self._istar = np.argmin(vals)
+        return vals[self._istar], self._pvals[self._istar]
+
+
 
 def two_sample_test(smp1, smp2, data_type = 'counts',
                          alt='two-sided', **kwargs) :
